@@ -15,3 +15,6 @@ pub use poll_semaphore::PollSemaphore;
 
 mod reusable_box;
 pub use reusable_box::ReusableBoxFuture;
+
+#[cfg(test)]
+mod tests;
